/*/////////////////////////////////////////////////////////////////////
							 filename:	count.cpp
						program name:	count.exe
								author:   sock
								  date:	31mar96
						 last update:	01apr96

									DESCRIPTION
	the count program  counts the sizes of the files in the current directory
	and all of its subdirectories and totals them.

*/

///////////////////////////INCLUDES/////////////////////////////////////
#include <io.h>
#include <dos.h>
#include <iostream.h>
#include <conio.h>
#include <stdio.h>
#include <dir.h>
#include <string.h>
#include "count.h"

/////////////////////////////DEFINES//////////////////////////////////////
#define VERNUM 	("1.00")
#define PROGNAME 	("count")

///////////////////////////FUNCTIONS//////////////////////////////////////

/*////////////////////////////////////////////////////////////////////////
function name: FindSubDirectories(char *path)

purpose: FindSubDirectories takes a DOS path name and searches it for
		 sub directories.

returns: >= 0  number of subdirectories found
			-1 failed to find parent directory.  this happens if path is "C:\"
			-2 failed to find path directory

//////////////////////////////////////////////////////////////////////////*/
int FindSubDirectories(char *path) {
	struct ffblk dirinfo;//structure to hold file information
	int findcheck;//did we find anything, 0 = yes
	int dirbit;//directory attribute bitt
	int subsfound = 0;//sub directories found this time
	char oldpath[MAXPATHL],//hold the original working directory
	pathnamebuf[MAXPATHL];//used to build the subdirectory path names

	//working directory at function start-up
	getcwd(oldpath, MAXPATHL);

	//change to the directory supplied to the function
	chdir(path);

	//first dir is always "."
	findcheck = findfirst("*.*", &dirinfo, FA_DIREC);

	//if for some reason the function fails on first directory
	if(findcheck != 0) {
		cout << "(.) not found" << endl;
		chdir(oldpath);
		return -1;
	 }

	//second dir is always ".."
	findcheck = findnext(&dirinfo);

	//if for some reason the function fails on second directory
	if(findcheck != 0) {
		cout << "(..) not found" << endl;
		chdir(oldpath);
		return -2;
	}

	//check for third dir entry
	findcheck = findnext(&dirinfo);

	//if no more dir found return 0
	if(findcheck != 0) {
		chdir(oldpath);
		return 0;
	}

	//execute loop while directory entries are found
	while(findcheck == 0) {

		//confusing, but make sure it's a directory
		dirbit = (dirinfo.ff_attrib & 16);

		//if it is, then add it to our list of subdirectory names
		if(dirbit == 16){
			strcpy(pathnamebuf, path);
			strcat(pathnamebuf, "\\");
			strcat(pathnamebuf, dirinfo.ff_name);
			strcpy(Subdir[Totalsubs].name, pathnamebuf);
			Totalsubs++;
			subsfound++;
		}

		//search for another directory entry. if none are found,
		//findcheck will not equal zero and loop will not
		  //execute again
		findcheck = findnext(&dirinfo);
	}

	//chg back to function start-up dir
	chdir(oldpath);

	//return the number of subdirectories found
	return subsfound;
}//end FindSubDirectories()

/*////////////////////////////////////////////////////////////////////////
function name: CountFiles(char *path)

purpose: sum the file sizes of the directory passed by path.

returns: the total size of all files in directory

//////////////////////////////////////////////////////////////////////////*/
long CountFiles(char *path){
	struct ffblk fileinfo;//struct to hold file information
	long dirsize = 0;//size of all files in directory
	int findcheck = -1;//did we find anything.  0 = yes.
	char oldpath[MAXPATHL];//working directory at function start-up

    //get current working directory
	getcwd(oldpath, MAXPATHL);

    //change to directory passed to function
    chdir(path);

	//search for any file
	findcheck = findfirst("\*.*", &fileinfo, 0);

	//if a file is found, add found file's size to total
	if(findcheck == 0) {
		dirsize += fileinfo.ff_fsize;
	}

	//execute loop while files are found
	while (findcheck == 0) {

		  //look for another file
		findcheck = findnext(&fileinfo);

		//if another file is found, add found file's size to total
		//if no file is found, findcheck will not equal 0 and
		  //loop will not execute again
		if(findcheck == 0) dirsize += fileinfo.ff_fsize;
	}

	//chg back to function start-up dir
	chdir(oldpath);

	//return size of all files in directory, note that if none are found,
	 //function returns 0
	 return dirsize;
}//end CountFiles()

///////////////////////////////////////MAIN//////////////////////////////////
int main(int argc)
{
	char oldpath[MAXPATHL];	//working directory at program start
	int found = -1;			//number of subdirectories found

	//if no command line arguments have been given
	if(argc == 1) {
		getcwd(oldpath,MAXPATHL);
	}
	else {
		cout << "usage: count" << endl;
		return -1;
	}

	cout << PROGNAME << " version number " << VERNUM << " by sock, 1996." \
			 << endl << endl;
	cout << "searching " << oldpath << "..." << endl;

	//search for subdirectories off the original path
	found = FindSubDirectories(oldpath);

	//execute while new subdirectories are found
	while(found > 0) {
		int substart;//where to search for new subdirectories

		//don't start at the beginning of the list again. start at the
		//end of the list minus the number of subdirectories we just
		//found.  that way we won't search subdirectories more than
		//once
		substart = Totalsubs - found;

		//search the number of subdirectories we just found
		for(int j = substart;j < Totalsubs;j++) {

			//get the number of subdirectories for this subdirectory
			//if there are no more for any of the subdirectories, then
			//found will not be greater than 0 and the while loop
				//will not execute agian
			found = FindSubDirectories(Subdir[j].name);
		}
	}//end while()
   
	//add our starting directory to the list of subdirectory names
	//this way we can just uase a for loop to count all the directory
	//contents
	strcpy(Subdir[Totalsubs].name, oldpath);

	//increment number subdirectories
	Totalsubs++;

	 //create variable for total space used
	long totdirsize = 0;

	//cycle through all the directories and count their sizes
	for(int j=0;j < Totalsubs;j++) {
		Subdir[j].size = CountFiles(Subdir[j].name);
		totdirsize += Subdir[j].size;
	}

   //set output formats
	float percentage = 0;
	cout.precision(2);
	cout.setf(ios::left, ios::adjustfield);
	cout.setf(ios::fixed, ios::floatfield);

	//cycle through all directories and output disk usage
	for(int k=0;k < Totalsubs;k++) {
		outlong(Subdir[k].size);
		if(Subdir[k].size > 999999L)
			cout << " bytes,\t";
		else
			cout << " bytes,\t\t";

		percentage = (float)Subdir[k].size/(float)totdirsize*100;
		cout << percentage << "%";
		cout << "\t" << Subdir[k].name << endl;
	}

	//let the user know what we've found.  note: Total subs includes the
	//original directory, so we subtract one to make it accurate
	cout << endl << "total number of subdirectories =  " << (Totalsubs - 1) << endl;
	cout << "total space used by subdirectories and their files =  ";
	outlong(totdirsize);
	cout << " bytes" << endl;

	return 0;
}//end main()
